<?php
abstract class db_adapter {
    var $prefix;
    var $query_count = 0;
    var $table_delimiter = '`';
    var $query_logger_callback = null;

    /**
     * Set the prefix that should be used.
     * e.g. joomla uses only tables starting with 'jos_', thus 
     * set_prefix('jos_') can be used for simplification.
     */
    function set_prefix($prefix) {
        $this->prefix = $prefix;
    }

    /**
     * Log and count one query
     */
    function log_query($query) {
        if (!is_null($this->query_logger_callback)) {
            call_user_func($this->query_logger_callback, $query);
        }
        $this->query_count++;
    }

    /**
     * Set database logging function.
     * ->set_logger('foo') calls global function foo($query)
     * ->set_logger(array('foo', 'bar')) calls static function foo::bar($query)
     */
    function set_logger($fn) {
        $this->query_logger_callback = $fn;
    }

    /**
     * Get the LIMIT part of an SQL query.
     * @param $limit = null means no limit.
     *   $limit = array('start' => 0, 'count' => 10)
     */
    public function get_limit($limit) {
        // prepare limit
        if ($limit == null) return '';
        assert('isset($limit[\'start\'])');
        $start = $limit['start'];
        assert('is_numeric($start)');
        assert('isset($limit[\'count\'])');
        $count = $limit['count'];
        assert('is_numeric($count)');

        // mysql version
        return " LIMIT $start, $count";
    }

    /**
     * Get the ORDER part of an SQL query
     */
    public function get_order($order) {
        if ($order == null) return '';
        return ' ORDER BY '.$order;
    }

    /**
     * Get the WHERE part of an SQL query
     * @param $order = null means no order.
     *   $order = 'id DESC'
     */
    public function get_where($where) {
        if ($where == null) return '';
        if (is_string($where)) return ' WHERE '.$where;
        if (is_array($where)) {
            if (count($where) == 0) return '';
            $result = '';
            foreach ($where as $k => $v) {
                if (strlen($result) > 0) $result .= ' AND ';
                $result .= $this->table_delimiter . mysql_escape_string($k);
                if (is_string($v)) {
                    $result .= $this->table_delimiter . ' = \''.$v.'\'';
                } elseif (is_numeric($v)) {
                    $result .= $this->table_delimiter . ' = '.$v;
                } elseif (is_bool($v)) {
                    $result .= $this->table_delimiter . ' = '.($v?'TRUE':'FALSE');
                } elseif (is_null($v)) {
                    $result .= $this->table_delimiter . ' IS NULL';
                } elseif ($v instanceof database_object) {
                    $result .= '_id'.$this->table_delimiter.' = '.($v->id);
                } else {
                    die('sql WHERE type fail');
                }
            }
            return ' WHERE '.$result;
        }
        die('$where must be null, a string or an array');
    }

    /**
     * Those abstract functions have to be overwritten by sub-classes like 
     * db_adodb and db_mysql.
     */
    abstract function query($query);
    abstract function fetch_columns($table);
    abstract function select_single($query);
    abstract function init();

}

