<?php
/**
 * AdoDB database adapter
 */
require_once('adodb/adodb.inc.php');

class db_adodb extends db_adapter {
    var $db = null;

    public function db_adodb($host_or_db, $user = null, $pass = null, $dbase = null) {
        $this->table_delimiter = '"';
        if ($host_or_db instanceof ADODB_postgres7) {
            // AdoDB object given => use this
            $this->db = $host_or_db;
            //$this->dbase = $host_or_db->database;
            return;
        }

        // host / user / pass / db given
        assert('is_string($host_or_db)');
        assert('is_string($user)');
        assert('is_string($pass)');
        assert('is_string($dbase)');

        // connect to postgres7
        $this->db = NewADOConnection("postgres7://" . $user . ":" .
            $pass . "@" . $host_or_db . "/" . $dbase);
    }

    public function init() {
    }

    public function query($query) {   
        $r = $this->db->Execute($query);
        if ($r == false) {
            // again, with debug
            $this->db->debug = true;
            $r = $this->db->Execute($query);
            assert('$r != false');
        }
        return $r;
    }

    public function query_row($query) {   
        $record = $this->db->GetRow($query);
        // TODO result check
        return $record;
    }

    /**
     *
     */
    public function get_limit($_limit) {
        // prepare limit
        if ($_limit == null) return '';
        assert('isset($_limit[\'start\'])');
        $start = $_limit['start'];
        assert('is_numeric($start)');
        assert('isset($_limit[\'count\'])');
        $count = $_limit['count'];
        assert('is_numeric($count)');

        // mysql version
        //return " LIMIT $start, $count";

        // postgres version
        return " LIMIT $count OFFSET $start";
    }

    public function query_all($query) {
        $all = $this->db->GetAll($query);
        if ($all === false) {
            // again with debug
            $this->db->debug = true;
            $all = $this->db->GetAll($query);
            die("SQL query failed.");
        }
        return $all;
    }

    private function auto_execute($table, $record, $action, $where = null) {
        // TODO AutoExecute does SELECT * before UPDATE => sux
        if (is_null($where)) {
            $r = $this->db->AutoExecute($table, $record, $action);
        } else {
            $r = $this->db->AutoExecute($table, $record, $action, $where); 
        }
        if ($r == false) {
            // again, with debug
            $this->db->debug = true;
            if (is_null($where)) {
                $r = $this->db->AutoExecute($table, $record, $action);
            } else {
                $r = $this->db->AutoExecute($table, $record, $action, $where); 
            }
            assert('$r != false /* query failed: AutoExecute '.$action.' on '.
                $table.' */');
        }
    }

    public function update($table, $record, $id) {
        $this->auto_execute($table, $record, 'UPDATE', 'id='.$id);
    }

    public function insert($table, $record) {
        if (sizeof($record) == 0) {
            // AdoDB failt wenn insert mit nur default parametern
            $query = "INSERT INTO ".$table." VALUES (DEFAULT)";
            $r = $this->query($query);
        } else {
            $this->auto_execute($table, $record, 'INSERT');
        }
        // geht nicht obwohl dokumentiert: 
        // $id = $this->db->Insert_ID();
        // assert('$id != false');
        $r = $this->db->GetRow('SELECT id FROM '.$table.' ORDER BY id '.
            'DESC LIMIT 1');
        assert('$r != false');
        $id = $r[0];
        return $id;
    }

    /**
     * Optimize this table. Call e.g. from Admin interface or once on long 
     * program run.
     */
    public static function optimize($table) {
        // Not implemented for AdoDB. Just do nothing
    }

    public function fetch_columns($table) {
        $columns = array();
        $result = $this->db->MetaColumns($table);
        if ($result === false) die('MetaColumns failed on '.$table.'. Is table '.$table.' existent in db?');
        foreach ($result as $row) {
                $columns[$row->name] = $this->from_adodb_type($row->type);
        }
        return $columns;
    }

    /** 
     * convert postgresql type to php type
     */
    private function from_adodb_type($type) {
        switch ($type) {
        case 'int2':
        case 'int4':
            return 'integer';
        case 'bool': 
            return 'bool';
        case 'float4':
            return 'float';
        case 'date':
            return 'date';
        case 'timestamp': 
            return 'datetime';
        case 'text':
        case 'bpchar':
        case 'varchar':
            return 'string';
        default: 
            //die($type);
            return 'string';
        }
    }

    /**
     * Select one single number from the datase (e.g. a count)
     */
    public function select_single($query) {
        $result = $this->db->GetAll($query);
        if ($result === false) {
            // again with debug
            $this->db->debug = true;
            $this->db->GetAll($query);
            die("SQL query failed.");
        }
        return $result[0][0];
    }
}

